"""
Config.py
February 13, 2011
updated: April 24, 2011

Reads and writes config files in a simple (key = value) or just (key) for boolean
properties, and notifies listeners when the values are changed.
"""

import errno
import logging
import os

from gi.repository import GObject
from threading import Lock

logger = logging.getLogger("Config")

class ConfigException(Exception):
  pass

class Config(GObject.GObject):
  """A config file"""
  __gsignals__ = {
      "value-changed": (GObject.SignalFlags.RUN_LAST, None, [str, str])
  }
  def __init__(self, path):
    GObject.GObject.__init__(self)

    self._path = os.path.expanduser(path)
    self._options = {}
    self._lock = Lock()

    self.load()

  def __contains__(self, key):
    """Returns true if this config has the given key."""
    return key in self._options

  def __iter__(self):
    """Returns an iterator over the keys of this Config object."""
    return iter(self._options)

  def load(self):
    """Loads a config file, if it exists."""
    try:
      f = open(self._path, "r")
    except:
      logger.warning("Failed to open config file: {0}".format(self._path))
      self._options = {}
      return

    try:
      line_number = 1
      for line in f:
        # Skip blank lines
        if line.strip() == "":
          continue

        item = line.split("=", 1)
        
        key = item[0].strip()
        if len(item) == 1:
          value = True
        else:
          value = item[1].strip()

        # Empty key
        if key == "":
          raise ConfigException("Empty key on line {0}: {1}".format(line_number, line.strip()))

        self._options[key] = value
        self.emit("value-changed", key, value)
        line_number += 1
    except:
      logger.exception("Error reading config file: {0}".format(self._path))
      self._options = {}

  def save(self):
    """Saves the config file, creating the destination directory if necessary."""
    # Create the destination directory
    dirname = os.path.dirname(self._path)
    try:
      os.makedirs(dirname, 0700)
    except os.error as e:
      if e.errno != errno.EEXIST:
        logger.error("Failed to create directory: {0}".format(dirname))
        return

    with open(self._path, "w") as f:
      for key, value in self._options.items():
        f.write("{0} = {1}\n".format(key, value))

  def get(self, key, default = ""):
    """Returns the value of the given key, or the default value."""
    if key in self._options:
      return self._options[key]
    else:
      return default

  def get_bool(self, key, default = False):
    """Returns the value of the given key as a boolean, or the default value if it fails
    to convert or does not exist."""
    if key not in self._options:
      return default
    value = self._options[key]
    if value == True or value == "True":
      return True
    elif value == False or value == "False":
      return False
    else:
      return default

  def get_int(self, key, default = 0):
    """Returns this value of the given key as an int, or the default value if it fails to
    convert or does not exist."""
    if key not in self._options:
      return default
    value = self._options[key]
    try:
      return int(value)
    except:
      return default

  def set(self, key, value, autosave = True):
    """Sets the value for the given key, saves the config file and notifies any
    listeners that the value has changed. If autosave is True, the config file
    will be saved after the value is set."""
    if not (isinstance(key, str) or isinstance(key, unicode)):
      raise ConfigException("Trying to set a non-string key: {0} = {1}".format(key, value))

    key = key.strip()
    if key == "":
      raise ConfigException("Trying to set empty key to value: {0}".format(value))

    with self._lock:
      self._options[key] = value
      if autosave:
        self.save()
    self.emit("value-changed", key, value)

  def set_if_unset(self, key, value, autosave = True):
    """Calls set(key, value) only if the key does not exist in this config."""
    if key not in self._options:
      self.set(key, value, autosave)

