"""
Theme.py
April 25, 2011

Classes for dealing with color themes
"""

import logging
import os

from gi.repository import Gdk, Gtk

import RoundedWidgets

from Config import Config

SCRIPT_PATH = os.path.dirname(os.path.abspath(__file__))

# TODO: Using local theme path while testing, should update to
# an actual theme path, and then fallback to the local theme
# path if running from source
SYSTEM_THEME_PATH = os.path.abspath(os.path.join(SCRIPT_PATH, "..", "themes"))

USER_THEME_PATH = "~/.config/recent-notifications/themes"

DEFAULT_BACKGROUND = "#eee"
DEFAULT_ITEM_BACKGROUND = "#555"
DEFAULT_ITEM_FOREGROUND = "#fff"
DEFAULT_LINK_COLOR = "#ea1"
DEFAULT_VISITED_LINK_COLOR = "#fd7"

logger = logging.getLogger("Theme")

gtk_style_template = """
style "message_list" {{
  bg[NORMAL]="{message_list_bg}"
}}

style "message_item" {{
  bg[NORMAL]="{message_item_bg}"
}}

style "message_label" {{
  fg[NORMAL]="{message_label_fg}"
  GtkLabel::link-color="{message_label_link}"
  GtkLabel::visited-link-color="{message_label_visited}"
}}

widget "*message_list*" style "message_list"
widget "*message_item*" style "message_item"
widget "*message_label*" style "message_label"
"""

class ColorException(Exception):
  pass

class Color(object):
  """Wraps a GdkColor"""
  MAX_VALUE = 65535.0
  def __init__(self, color_string):
    """Creates a new Color object from the given color string.
    Uses Gdk.color_parse to parse the string."""
    result, color = Gdk.color_parse(color_string)
    if result != True:
      raise ColorException("Failed to parse color: {0}".format(color_string))

    self._color = color
    self._red = self._float_color(color.red)
    self._green = self._float_color(color.green)
    self._blue = self._float_color(color.blue)

  def __str__(self):
    """Returns a hex string version of the color"""
    return self._color.to_string()

  def _float_color(self, value):
    """Returns the color value as a float from 0.0 to 1.0"""
    return float(value) / Color.MAX_VALUE

  @property
  def gdk_color(self):
    """Returns the gdk color"""
    return self._color

  @property
  def red(self):
    """Returns the red component of the color as a float from 0.0 to 1.0"""
    return self._red

  @property
  def green(self):
    """Returns the green component of the color as a float from 0.0 to 1.0"""
    return self._green

  @property
  def blue(self):
    """Returns the blue component of the color as a float from 0.0 to 1.0"""
    return self._blue
    
  def set_fg_normal(self, widget):
    """Sets the NORMAL foreground color of the given widget to this color"""
    widget.modify_fg(Gtk.StateType.NORMAL, self._color)

  def set_bg_normal(self, widget):
    """Sets the NORMAL background color of the given widget to this color"""
    widget.modify_bg(Gtk.StateType.NORMAL, self._color)

class ThemeException(Exception):
  pass

class Theme(object):
  """Represents a color theme.
  TODO: promote to a gobject so it can emit theme-changed signals"""
  def __init__(self, filename):
    """Loads a theme with the given filename, or creates a new theme if not found.
    First searches the user theme directory ~/.config/recent-notifications/themes/
    then searches the system theme directory. If no theme is found, the new
    theme is created in the user theme directory."""

    self._filename = filename

    user_path = os.path.expanduser(USER_THEME_PATH)
    user_file = os.path.join(user_path, filename)
    system_file = os.path.join(SYSTEM_THEME_PATH, filename)

    if os.path.exists(user_file):
      logger.debug("Loading theme from user path: {0}".format(user_file))
      self._path = user_path
      self._config = Config(user_file)
    elif os.path.exists(system_file):
      logger.debug("Loading theme from system path: {0}".format(system_file))
      self._path = system_file
      self._config = Config(system_file)
    else:
      logger.debug("Creating new user theme: {0}".format(user_file))
      self._path = user_path
      self._config = Config(user_file)

    # Set default values if the properties are missing
    self._config.set_if_unset("background", DEFAULT_BACKGROUND)
    self._config.set_if_unset("item_background", DEFAULT_ITEM_BACKGROUND)
    self._config.set_if_unset("item_foreground", DEFAULT_ITEM_FOREGROUND)
    self._config.set_if_unset("link_color", DEFAULT_LINK_COLOR)
    self._config.set_if_unset("visited_link_color", DEFAULT_VISITED_LINK_COLOR)

  def apply_gtk_styles(self):
    """Applies settings to the global gtk style"""
    bg = self.get_background()
    item_bg = self.get_item_background()
    item_fg = self.get_item_foreground()
    link_color = self.get_color("link_color")
    visited_link_color = self.get_color("visited_link_color")

    Gtk.rc_parse_string(gtk_style_template.format(
      message_list_bg=bg,
      message_item_bg=item_bg,
      message_label_fg=item_fg,
      message_label_link=link_color,
      message_label_visited=visited_link_color))

    # Hack until I can figure out how to get the bg color from the theme
    RoundedWidgets.RoundedBox.BACKGROUND_COLOR = item_bg

  def get_name(self):
    """Returns the name of this theme, or if unset, the filename"""
    return self._config.get("name", self._filename)

  def get_color(self, key):
    """Returns a new color for the given property"""
    if key not in self._config:
      raise ThemeException("Invalid property: {0}".format(key))

    color_string = self._config.get(key)
    return Color(color_string)

  def get_background(self):
    """Returns the background property as a Color"""
    return self.get_color("background")

  def get_item_background(self):
    """Returns the item_background property as a Color"""
    return self.get_color("item_background")

  def get_item_foreground(self):
    """Returns the item_foreground property as a Color"""
    return self.get_color("item_foreground")
